/* Copyright John-Philip "Seriema" Johansson (seriema@users.sourceforge.net)
 * released under the MIT license */
using System;
using System.Runtime.InteropServices;
using System.Collections;

namespace ThumbViewConnector
{
	/// <summary>
	/// Connects to a ThumbView Extractor and gives access to its exported functions. 
	/// </summary>
	public class ExtractorConnector
	{
		/// <summary>
		/// An enumeration of pixel formats supported by the Extractors.
		/// </summary>
		public enum PixelFormat { Palette = 8, RGB24 = 24, RGBA32 = 32 };

		/// <summary>
		/// The ThumbView Extractor DLL that should be used.
		/// In this sample we will use Beermap, "BeerPack.dll".
		/// To use DevIL just change the value to "DevILPack.dll".
		/// </summary>
		private const string dllName = "BeerPack.dll";

		#region Extractor information

		/// <summary>
		/// The name of the called ThumbView Extractor.
		/// </summary>
		/// <returns>The name of this Extractor as a null terminated string.</returns>
		[DllImport(dllName, EntryPoint="Name")]
		private static extern string NameCall();
		public static string Name  { get { return NameCall(); } }

		/// <summary>
		/// The version of the called ThumbView Extractor.
		/// </summary>
		/// <returns>The version number as a null terminated string. It is correctly string-comparable to other versions.</returns>
		[DllImport(dllName, EntryPoint="Version")]
		private static extern string VersionCall();
		public static Version Version { get { return new Version(VersionCall()); } }

		/// <summary>
		/// The filetypes are supported by the ThumbView Extractor.
		/// </summary>
		/// <returns>A nullterminated array of pointers to supported file extensions, each file extension is a lowercase string and excludes the dot ('.') character.</returns>
		[DllImport(dllName, EntryPoint="SupportedFiletypes")]
		private static extern IntPtr SupportedFiletypesCall();

		/// <summary>
		/// Wraps SupportedFiletypes so one gets a Drawing.Bitmap instead of a IntPtr.
		/// </summary>
		/// <returns>A array of strings to supported file extensions, each file extension is a lowercase string and excludes the dot ('.') character.</returns>
		public static string[] SupportedFiletypes
		{
			get
			{
				ArrayList strings = new ArrayList(); // Use System.Collections.Generic.List if you're on .NET 2.0
				IntPtr p = SupportedFiletypesCall();

				unsafe 
				{
					int* ptr = (int*)p.ToPointer();

					while( *ptr != 0 ) 
					{
						string s = Marshal.PtrToStringAnsi(new IntPtr(*ptr));
						strings.Add(s);

						p = new IntPtr(ptr + 1);
						ptr++;
					}
				}

				return (string[])strings.ToArray(typeof(string));
			}
		}

		#endregion

		#region Image processing and information

		/// <summary>
		/// Wraps ExtractThumbnailCall so one gets a Drawing.Bitmap instead of a IntPtr.
		/// </summary>
		/// <param name="Path">Zero terminated file path to the image.</param>
		/// <param name="RecomendedBPP">Recommended color depth in units of bits per pixel.</param>
		/// <param name="RequiredSize">Desired width and height of the image.</param>
		/// <returns>A handle to a System.Drawing.Bitmap on success, NULL on failure.</returns>
		public static System.Drawing.Bitmap ExtractThumbnail(string Path, PixelFormat RecomendedBPP, ref System.Drawing.Size RequiredSize)
		{
			IntPtr thumbnail = ExtractThumbnailCall( Path, RecomendedBPP, ref RequiredSize);
			if( thumbnail == IntPtr.Zero )
				return null;

			return System.Drawing.Image.FromHbitmap( thumbnail );
		}
		[DllImport(dllName, EntryPoint="ExtractThumbnail")]
		private static extern IntPtr ExtractThumbnailCall(string Path, PixelFormat RecomendedBPP, ref System.Drawing.Size RequiredSize);


		/// <summary>
		/// Collects basic information about the givem image.
		/// </summary>
		/// <param name="Path">Zero terminated file path to the image.</param>
		/// <param name="BPP">Original color depth in units of bits per pixel.</param>
		/// <param name="Size">Original width and height of the image.</param>
		/// <returns>False on error, otherwise true.</returns>
		[DllImport(dllName)]
		public static extern bool GetInfo(string Path, out PixelFormat BPP, out System.Drawing.Size Size);

		#endregion
	}
}
